/**************************** CPPHeaderFile ***************************

* FileName [CEDagVerifier.h]

* PackageName [main]

* Synopsis [Header file for CEDagVerifier class.]

* Description [This class verifies if the projection of the counter
* example on a program component is valid.]

* SeeAlso [CEDagVerifier.cpp]

* Author [Sagar Chaki]

* Copyright [ Copyright (c) 2002 by Carnegie Mellon University.  All
* Rights Reserved. This software is for educational purposes only.
* Permission is given to academic institutions to use, copy, and
* modify this software and its documentation provided that this
* introductory message is not removed, that this software and its
* documentation is used for the institutions' internal research and
* educational purposes, and that no monies are exchanged. No guarantee
* is expressed or implied by the distribution of this code. Send
* bug-reports and/or questions to: chaki+@cs.cmu.edu. ]

**********************************************************************/

#ifndef __CEDAGVERIFIER_H__
#define __CEDAGVERIFIER_H__

//other classes needed
class ProcManager;
class ImplState;
class ConcCEDag;

/*********************************************************************/
//the CE dag verifier
/*********************************************************************/
class CEDagVerifier
{
 private:
  //the procedure manager we are concerned with
  ProcManager &procManager;

  //flag to indicate if a spurious path was discovered in the concrete
  //CE dag
  bool spuriousCE;

  //top-level routines
  int RunFsp();
  int RunLtl();

  void VerifyConcreteDag(const ConcCEDag &concDag);
  void UpdatePrePost(set<ContLoc*> &expLocs,set<Expr> &expSet,const int node,const ConcCEDag &concDag,const bool pre) const;
  set<Expr> UpdateSP(const set<Expr> &old,const Expr &lhs,const Expr &rhs) const;
  set<Expr> UpdateWP(const set<Expr> &old,const Expr &lhs,const Expr &rhs) const;
  void CreateSpuriousAbsLtsCEInfo(const int node,const set<const ImplState*> &impls,const ConcCEDag &concDag) const;

 public:
  static const int CE_VALID;
  static const int CE_INVALID_DAG;
  static const int CE_INVALID_PATH;

  CEDagVerifier(ProcManager &p) : procManager(p) { 
    spuriousCE = false; 
  }
  CEDagVerifier(const CEDagVerifier &rhs) : procManager(rhs.procManager) { 
    spuriousCE = false; 
  }

  int Run();
};

#endif //__CEDAGVERIFIER_H__

/*********************************************************************/
//end of CEDagVerifier.h
/*********************************************************************/
