/**************************** CPPHeaderFile ***************************

* FileName [ImplState.h]

* PackageName [main]

* Synopsis [Header file for ImplState class.]

* Description [This class encapsulates a state of the implementation
* LTS.]

* SeeAlso [ImplState.cpp]

* Author [Sagar Chaki]

* Copyright [ Copyright (c) 2002 by Carnegie Mellon University.  All
* Rights Reserved. This software is for educational purposes only.
* Permission is given to academic institutions to use, copy, and
* modify this software and its documentation provided that this
* introductory message is not removed, that this software and its
* documentation is used for the institutions' internal research and
* educational purposes, and that no monies are exchanged. No guarantee
* is expressed or implied by the distribution of this code. Send
* bug-reports and/or questions to: chaki+@cs.cmu.edu. ]

**********************************************************************/

#ifndef __IMPLSTATE_H__
#define __IMPLSTATE_H__

//other classes needed
class ImplState;
class Action;
class ContLoc;
class PredSet;

/*********************************************************************/
//a set of implementation states
/*********************************************************************/
class ImStSet
{
 private:
  bool all;
  set<const ImplState*> ele;

 public:
  ImStSet() { all = false; }
  ImStSet(const set<const ImplState*> &e,const ProcManager &procManager);
  ~ImStSet() {}
  ImStSet(const ImStSet &rhs) { *this = rhs; }
  const ImStSet &operator = (const ImStSet &rhs);
  bool operator == (const ImStSet &rhs) const;
  bool operator < (const ImStSet &rhs) const;

  static ImStSet GetComplete();
  const set<const ImplState*> &GetEle(const ProcManager &procManager) const;
  ImStSet GetPredsOnAction(const Action &act,const ProcManager &procManager) const;
  ImStSet GetSuccsOnAction(const Action &act,const ProcManager &procManager) const;
  void Intersect(const ImStSet &rhs);
  const ImplState* Pick(const ProcManager &procManager) const;
  bool Contains(const ImplState *arg) const;
  bool Contains(const ContLoc *arg) const;
  bool Empty() const;
  void Display(const ProcManager &procManager) const;
};

/*********************************************************************/
//the implementation state
/*********************************************************************/
class ImplState
{
 private:
  //the id of the control location
  short contLocId;
  
  //the predicate valuation
  PVal predVal;
  
  //the index of the state of the abstracting spec LTS
  short specStateId;
  
  //the index of the info about the abstracting lts
  char specInfoId;
  
  //the index of the abstract state corresponding to this
  //implementation state
  int absId;

  list< pair<const ImplState*,Action> > GetSuccsAndActionsFinal() const;
  list< pair<const ImplState*,Action> > GetSuccsAndActionsReturn() const;
  list< pair<const ImplState*,Action> > GetSuccsAndActionsBranch() const;
  list< pair<const ImplState*,Action> > GetSuccsAndActionsAssign() const;
  list< pair<const ImplState*,Action> > GetSuccsAndActionsCall() const;

  void HandleConstraint(ContLoc *succ,const Expr &cons,const set<Action> &acts,
			list< pair<const ImplState*,Action> > &ret) const;
  void HandleAssign(ContLoc *succ,const list<Expr> &lhsList,const list<Expr> &rhsList,
		    const set<Action> &acts,list< pair<const ImplState*,Action> > &ret,
		    const short fsid = -1,const short fiid = -1) const;
  void AddSuccAndActions(list< pair<const ImplState*,Action> > &ret,const ImplState *succ,const set<Action> &actions) const;
  bool PossibleNextValuation(const list<Expr> &lhsList,const list<Expr> &rhsList,const PredSet &oldPreds,
			     const PVal &oldVal,const PredSet &newPreds,const PVal &newVal) const;

 public:
  ImplState() { assert(false); }
  ImplState(short c,const PVal &p,short ss,char si);
  ImplState(const ImplState &rhs);
  const ImplState &operator = (const ImplState &rhs);

  string ToString() const;

  set<Expr> GetPropositions() const;
  list< pair<const ImplState*,Action> > GetSuccsAndActions() const;
  set<const ImplState*> GetSuccsOnAction(const Action &act) const;
  set<const ImplState*> GetPredsOnAction(const Action &act) const;
  set<Action> GetActions() const;
  string GetKey() const;
  const PVal &GetPredVal() const { return predVal; }
  short GetSpecStateId() const { return specStateId; }
  short GetSpecInfoId() const { return specInfoId; }
  size_t GetProcManagerId() const;
  ContLoc *GetContLoc() const;
  int GetAbsId() const { return absId; }

  void SetAbsId(int a) { absId = a; }
};

#endif //__IMPLSTATE_H__

/*********************************************************************/
//end of ImplState.h
/*********************************************************************/
