/**************************** CPPHeaderFile ***************************

* FileName [LtsInfo.h]

* PackageName [main]

* Synopsis [Header file for LtsInfo class.]

* Description [This class encapsulates a labeled transition system.]

* SeeAlso [LtsInfo.cpp]

* Author [Sagar Chaki]

* Copyright [ Copyright (c) 2002 by Carnegie Mellon University.  All
* Rights Reserved. This software is for educational purposes only.
* Permission is given to academic institutions to use, copy, and
* modify this software and its documentation provided that this
* introductory message is not removed, that this software and its
* documentation is used for the institutions' internal research and
* educational purposes, and that no monies are exchanged. No guarantee
* is expressed or implied by the distribution of this code. Send
* bug-reports and/or questions to: chaki+@cs.cmu.edu. ]

**********************************************************************/

#ifndef __LTSINFO_H__
#define __LTSINFO_H__

//other classes needed
class Action;
class LtsTrans;

class LtsInfo
{
 private:
  //the name of the LTS - this is also the name of the initial state
  string name;
  //whether the LTS is useful - if it contains a return or assign
  //action or an action common to the specification LTS
  bool useful;
  //the set of states reachable from the initial state
  set<string> states;
  //the map of states to indices
  map<string,int> statesToIds;
  //the set of actions
  set<Action> actions;
  //the transition relation
  set<LtsTrans> trans;
  //the index of the lts info the global vector
  char index;
  //the action successor map
  map< pair<string,Action>,set<string> > succMap;
  //the action predecessor map
  map< pair<string,Action>,set<string> > predMap;
  
  set<Action> ComputeReturnActions() const;
  set<Action> ComputeAssignActions() const;
  
 public:
  LtsInfo();
  LtsInfo(const string &n,const set<string> &s,const set<Action> &a,const set<LtsTrans> &t);
  LtsInfo(const LtsInfo &rhs);
  const LtsInfo &operator = (const LtsInfo &rhs);
  
  string GetName() const { return name; }
  string GetInitState() const { return name; }
  bool IsUseful() const { return useful; }
  char GetIndex() const { return index; }
  const set<string> &GetStates() const;
  const set<Action> &GetActions() const;
  
  void SetIndex(const char i) { index = i; }
  
  void ComputeUseful();
  set<Expr> GetRetActExprs() const;
  set<Action> GetReturnActions(const Expr &expr,const list<string> &params,
			       const set<Expr> &context) const;
  set<Action> GetAssignActions(const Expr &lhs,const Expr &rhs,
			       const list<string> &params,
			       const set<Expr> &context) const;
  list<LtsTrans> GetOutTrans(const string &source) const;
  set<string> GetSuccsOnAction(const string &source,const Action &action) const;
  set<string> GetSuccsOnAction(const int source,const Action &action) const;
  set<string> GetPredsOnAction(const string &source,const Action &action) const;
  set<string> GetPredsOnAction(const int source,const Action &action) const;
  int GetStateId(const string &s) const;
  string GetIdState(const int id) const;
  void Draw() const;
};

#endif //__LTSINFO_H__

/*********************************************************************/
//end of LtsInfo.h
/*********************************************************************/
