D			[0-9]
L			[a-zA-Z_]
UL                      [A-Z_]
LL                      [a-z_]
H			[a-fA-F0-9]
E			[Ee][+-]?{D}+
FS			(f|F|l|L)
IS			(u|U|l|L)*

%{
#include <cstdio>
#include <cassert>
#include <string>
#include <set>
#include <list>
#include <map>
#include <vector>
using namespace std;

#include "CSymbolTable.h"
#include "Node.h"
#include "Action.h"
#include "Database.h"
#include "StdcParser.h"

/*********************************************************************/
//global shutdown method defined in arc.cpp needed
/*********************************************************************/
__attribute__ ((noreturn)) void GlobalShutdown(int sig);

void count();
int check_type();
void alias();
void attribute();

//counters
int asmCount;
int commentCount;
%}

%option noyywrap
%x STDC SPEC ASM COMMENT

%%
         /* set the appropriate start condition */ 
         if(CSymbolTable::fileType == CSymbolTable::STDC_FILE) BEGIN(STDC);
         else if(CSymbolTable::fileType == CSymbolTable::SPEC_FILE) BEGIN(SPEC);
         else { printf("unknown file type : %d\n",CSymbolTable::fileType); GlobalShutdown(-1); }

<STDC,SPEC,COMMENT>"/*" {
  if((YY_START == STDC) || (YY_START == SPEC)) {
    BEGIN(COMMENT);
    commentCount = 1;
  } else if(YY_START == COMMENT) {
    commentCount++;
  } else assert(false);
}
<COMMENT>"*/" {
  commentCount--;
  if(commentCount == 0) {
    BEGIN((CSymbolTable::fileType == CSymbolTable::STDC_FILE) ? STDC : SPEC);
  }
}
<COMMENT>[^/*]*                                     { /* comment body - ignore */ }

<STDC,SPEC>"//"(.*)                                 { /* CPP comment - ignore */ }
<STDC>^("#"(.*))                                    { /* line beginning with '#' - ignore */ }

<STDC>"auto"			                    { count(); return(MAGIC_AUTO); }
<STDC>"break"			                    { count(); return(MAGIC_BREAK); }
<STDC>"__builtin_va_list"                           { count(); return(MAGIC_VOID); }
<STDC>"case"			                    { count(); return(MAGIC_CASE); }
<STDC>"char"			                    { count(); return(MAGIC_CHAR); }
<STDC>"const"|"__const"|"__const__"                 { count(); return(MAGIC_CONST); }
<STDC>"continue"		                    { count(); return(MAGIC_CONTINUE); }
<STDC>"default"		                            { count(); return(MAGIC_DEFAULT); }
<STDC>"do"			                    { count(); return(MAGIC_DO); }
<STDC>"double"		                            { count(); return(MAGIC_DOUBLE); }
<STDC>"else"			                    { count(); return(MAGIC_ELSE); }
<STDC>"enum"			                    { count(); return(MAGIC_ENUM); }
<STDC>"__extension__"                               { /* ignore */ }
<STDC>"extern"		                            { count(); return(MAGIC_EXTERN); }
<STDC>"float"			                    { count(); return(MAGIC_FLOAT); }
<STDC>"for"			                    { count(); return(MAGIC_FOR); }
<STDC>"goto"			                    { count(); return(MAGIC_GOTO); }
<STDC>"if"			                    { count(); return(MAGIC_IF); }
<STDC>"inline"|"__inline"|"__inline__"              { /* ignore */ }
<STDC>"int"			                    { count(); return(MAGIC_INT); }
<STDC>"long"			                    { count(); return(MAGIC_LONG); }
<STDC>"register"		                    { count(); return(MAGIC_REGISTER); }
<STDC,SPEC>"return"		                    { count(); return(MAGIC_RETURN); }
<STDC>"short"			                    { count(); return(MAGIC_SHORT); }
<STDC>"signed"|"__signed__"	                    { count(); return(MAGIC_SIGNED); }
<STDC>"sizeof"		                            { count(); return(MAGIC_SIZEOF); }
<STDC>"static"		                            { count(); return(MAGIC_STATIC); }
<STDC>"struct"		                            { count(); return(MAGIC_STRUCT); }
<STDC>"switch"		                            { count(); return(MAGIC_SWITCH); }
<STDC>"typedef"		                            { count(); CSymbolTable::inTypedef = true; return(MAGIC_TYPEDEF); }
<STDC>"typeof"|"__typeof"|"__typeof__"              { count(); return(MAGIC_TYPEOF); }
<STDC>"union"			                    { count(); return(MAGIC_UNION); }
<STDC>"unsigned"		                    { count(); return(MAGIC_UNSIGNED); }
<STDC>"void"			                    { count(); return(MAGIC_VOID); }
<STDC>"volatile"|"__volatile"|"__volatile__"        { count(); return(MAGIC_VOLATILE); }
<STDC>"while"			                    { count(); return(MAGIC_WHILE); }

<STDC>"__attribute__"[ \t\v\n\f]*"("[ \t\v\n\f]*"(" { attribute(); }

<STDC>"asm"|"__asm"|"__asm__"                       { count(); BEGIN(ASM); asmCount = 0; }
<ASM>[^;\"()]+                                      { count(); /* argument of asm directive - ignore */ }

<STDC>"#pragma"(.*)                                 { /* ignore */ }
<STDC>"__restrict"                                  { /* ignore */ }
<STDC>"internal_function"                           { /* ignore */ }
<STDC>"strong_alias"                                { alias(); }
<STDC>"weak_alias"                                  { alias(); }

<SPEC>"predicate"                                   { count(); return(MAGIC_PREDICATE); }
<SPEC>"auxiliary"                                   { count(); return(MAGIC_AUXILIARY); }
<SPEC>"inline"                                      { count(); return(MAGIC_INLINE); }
<SPEC>"alias"                                       { count(); return(MAGIC_ALIAS); }
<SPEC>"abstract"                                    { count(); return(MAGIC_ABSTRACT); }
<SPEC>".."                                          { count(); return(MAGIC_DOT_DOT); }
<SPEC>"range"                                       { count(); return(MAGIC_RANGE); }
<SPEC>"set"                                         { count(); return(MAGIC_SET); }
<SPEC>"STOP"                                        { count(); return(MAGIC_STOP); }
<SPEC>"when"                                        { count(); return(MAGIC_WHEN); }
<SPEC>"property"                                    { count(); return(MAGIC_PROPERTY); }
<SPEC>"cprog"                                       { count(); return(MAGIC_CPROG); }
<SPEC>"cproc"                                       { count(); return(MAGIC_CPROC); }
<SPEC>"context"                                     { count(); return(MAGIC_CONTEXT); }
<SPEC>"X"                                           { count(); return(MAGIC_NEXT_TIME); }
<SPEC>"G"|"[]"                                      { count(); return(MAGIC_GLOBALLY); }
<SPEC>"F"|"<>"                                      { count(); return(MAGIC_FUTURE); }
<SPEC>"U"                                           { count(); return(MAGIC_UNTIL); }
<SPEC>"R"                                           { count(); return(MAGIC_RELEASE); }

<STDC>{L}({L}|{D})*     		{ count(); CSymbolTable::buffer = Stdctext; return(check_type()); }
<SPEC>[$]({L}|{D})*     		{ count(); CSymbolTable::buffer = Stdctext; return(MAGIC_IDENTIFIER); }
<SPEC>{UL}({L}|{D})*     		{ count(); CSymbolTable::buffer = Stdctext; return(MAGIC_UPPER_ID); }
<SPEC>{LL}({L}|{D})*     		{ count(); CSymbolTable::buffer = Stdctext; return(MAGIC_LOWER_ID); }

<STDC,SPEC>0[xX]{H}+{IS}?		{ count(); CSymbolTable::longbuf = strtol(Stdctext,NULL,0); return(MAGIC_INT_CONSTANT); }
<STDC,SPEC>0{D}+{IS}?		        { count(); CSymbolTable::longbuf = strtol(Stdctext,NULL,0); return(MAGIC_INT_CONSTANT); }
<STDC,SPEC>{D}+{IS}?	        	{ count(); CSymbolTable::longbuf = strtol(Stdctext,NULL,0); return(MAGIC_INT_CONSTANT); }
<STDC,SPEC>L?'(\\.|[^\\'])+' 	        { count(); CSymbolTable::buffer = Stdctext; return(MAGIC_CONSTANT); }

<STDC,SPEC>{D}+{E}{FS}?		        { count(); CSymbolTable::buffer = Stdctext; return(MAGIC_CONSTANT); }
<STDC,SPEC>{D}*"."{D}+({E})?{FS}?	{ count(); CSymbolTable::buffer = Stdctext; return(MAGIC_CONSTANT); }
<STDC,SPEC>{D}+"."{D}*({E})?{FS}?	{ count(); CSymbolTable::buffer = Stdctext; return(MAGIC_CONSTANT); }

<STDC,SPEC,ASM>L?\"(\\.|[^\\"])*\"   { 
                                       count(); 
                                       if((YY_START == STDC) || (YY_START == SPEC)) {
                                         CSymbolTable::buffer = Stdctext; 
                                         return(MAGIC_STRING_LITERAL);
                                       } else if(YY_START == ASM) {}
                                       else assert(false);
                                     }

<STDC>"..."			{ count(); return(MAGIC_ELLIPSIS); }
<STDC>">>="			{ count(); return(MAGIC_RIGHT_ASSIGN); }
<STDC>"<<="			{ count(); return(MAGIC_LEFT_ASSIGN); }
<STDC>"+="			{ count(); return(MAGIC_ADD_ASSIGN); }
<STDC>"-="			{ count(); return(MAGIC_SUB_ASSIGN); }
<STDC>"*="			{ count(); return(MAGIC_MUL_ASSIGN); }
<STDC>"/="			{ count(); return(MAGIC_DIV_ASSIGN); }
<STDC>"%="			{ count(); return(MAGIC_MOD_ASSIGN); }
<STDC>"&="			{ count(); return(MAGIC_AND_ASSIGN); }
<STDC>"^="			{ count(); return(MAGIC_XOR_ASSIGN); }
<STDC>"|="			{ count(); return(MAGIC_OR_ASSIGN); }
<STDC,SPEC>">>"			{ count(); return(MAGIC_RIGHT_OP); }
<STDC,SPEC>"<<"			{ count(); return(MAGIC_LEFT_OP); }
<STDC>"++"			{ count(); return(MAGIC_INC_OP); }
<STDC>"--"			{ count(); return(MAGIC_DEC_OP); }
<STDC,SPEC>"->"			{ count(); return(MAGIC_PTR_OP); }
<STDC,SPEC>"&&"			{ count(); return(MAGIC_AND_OP); }
<STDC,SPEC>"||"			{ count(); return(MAGIC_OR_OP); }
<STDC,SPEC>"<="			{ count(); return(MAGIC_LE_OP); }
<STDC,SPEC>">="			{ count(); return(MAGIC_GE_OP); }
<STDC,SPEC>"=="			{ count(); return(MAGIC_EQ_OP); }
<STDC,SPEC>"!="			{ count(); return(MAGIC_NE_OP); }
<STDC,SPEC,ASM>";"	{ 
                          count(); 
                          if((YY_START == STDC) || (YY_START == SPEC)) return(';');
                          else if(YY_START == ASM) {
                            if(asmCount == 0) { 
                              BEGIN((CSymbolTable::fileType == CSymbolTable::STDC_FILE) ? STDC : SPEC); 
                              return(';'); 
                            } 
                          } else assert(false);
                        }
<STDC,SPEC>("{"|"<%")		{ count(); CSymbolTable::PushScope(); return('{'); }
<STDC,SPEC>("}"|"%>")		{ count(); CSymbolTable::PopScope(); return('}'); }
<STDC,SPEC>","			{ count(); return(','); }
<STDC,SPEC>":"			{ count(); return(':'); }
<STDC,SPEC>"="			{ count(); return('='); }
<STDC,SPEC,ASM>"("      {
                          count(); 
			  if((YY_START == STDC) || (YY_START == SPEC)) return('(');
			  else if(YY_START == ASM) asmCount++;
			  else assert(false);
			} 
<STDC,SPEC,ASM>")"      { 
                          count(); 
			  if((YY_START == STDC) || (YY_START == SPEC)) return(')');
			  else if(YY_START == ASM) asmCount--; 
			  else assert(false);
			}
<STDC,SPEC>("["|"<:")		{ count(); return('['); }
<STDC,SPEC>("]"|":>")		{ count(); return(']'); }
<STDC,SPEC>"."			{ count(); return('.'); }
<STDC,SPEC>"&"			{ count(); return('&'); }
<STDC,SPEC>"!"			{ count(); return('!'); }
<STDC,SPEC>"~"			{ count(); return('~'); }
<STDC,SPEC>"-"			{ count(); return('-'); }
<STDC,SPEC>"+"			{ count(); return('+'); }
<STDC,SPEC,COMMENT>"*"       	{ count(); if((YY_START == STDC) || (YY_START == SPEC)) { count(); return('*'); } }
<STDC,SPEC,COMMENT>"/"       	{ count(); if((YY_START == STDC) || (YY_START == SPEC)) { count(); return('/'); } }
<STDC,SPEC>"%"			{ count(); return('%'); }
<STDC,SPEC>"<"			{ count(); return('<'); }
<STDC,SPEC>">"			{ count(); return('>'); }
<STDC,SPEC>"^"			{ count(); return('^'); }
<STDC,SPEC>"|"			{ count(); return('|'); }
<STDC,SPEC>"?"			{ count(); return('?'); }

<STDC,SPEC>[ \t\v\n\f\r]	{ count(); }
<*>.    			{ printf("unknown character ...\n"); GlobalShutdown(-1); }
%%

void alias()
{
  char c;
  while ((c = yyinput()) != ')' && c != 0);
}

void attribute()
{
  char c;
  int count = 2;
  while(true) {
    c = yyinput();
    if(c == 0) break;
    if(c == '(') count++;
    if(c == ')') {
      count--;
      if(count == 0) break;
    }
  }
}

int column = 0;

void count()
{
  int i;
  
  for (i = 0; Stdctext[i] != '\0'; i++)
    if (Stdctext[i] == '\n')
      column = 0;
    else if (Stdctext[i] == '\t')
      column += 8 - (column % 8);
    else
      column++;
  
  if(Database::PARSE_ECHO) ECHO;
}

int check_type()
{
  if(CSymbolTable::NameInScope(Stdctext)) {
    return MAGIC_TYPE_NAME;
  } else {
    return MAGIC_IDENTIFIER;
  } 
}

//to prevent the warning message that yyunput() is unused
void StdcDummy() { yyunput(0,NULL); }

